<?php

namespace Drupal\Tests\revision_menu_trail\Functional;

use Drupal\menu_link_content\Entity\MenuLinkContent;
use Drupal\Tests\BrowserTestBase;
use Drupal\Tests\content_moderation\Traits\ContentModerationTestTrait;
use Drupal\Tests\system\Functional\Menu\AssertMenuActiveTrailTrait;
use Drupal\Tests\taxonomy\Traits\TaxonomyTestTrait;
use Drupal\workflows\Entity\Workflow;

/**
 * Tests that the menu block shows for revision paths.
 *
 * @group revision_menu_trail
 */
class RevisionMenuTrailTest extends BrowserTestBase {

  use AssertMenuActiveTrailTrait;
  use ContentModerationTestTrait;
  use TaxonomyTestTrait;

  /**
   * {@inheritdoc}
   */
  protected static $modules = [
    'block',
    'content_moderation',
    'menu_link_content',
    'node',
    'revision_menu_trail',
    'taxonomy',
    'views',
  ];

  /**
   * {@inheritdoc}
   */
  protected $defaultTheme = 'starterkit_theme';

  /**
   * The editorial workflow entity.
   *
   * @var \Drupal\workflows\Entity\Workflow
   */
  protected Workflow $workflow;

  /**
   * {@inheritdoc}
   */
  protected function setUp(): void {
    parent::setUp();
    $this->drupalPlaceBlock('system_menu_block:main', ['depth' => 2]);
    $this->workflow = $this->createEditorialWorkflow();

    $this->drupalLogin($this->drupalCreateUser([
      'administer nodes',
      'administer taxonomy',
      'use editorial transition create_new_draft',
      'use editorial transition publish',
      'view latest version',
      'view any unpublished content',
    ]));
  }

  /**
   * Test that the active trail is added for node revisions.
   */
  public function testNodeRevisionMenuTrail() {
    $this->drupalCreateContentType(['type' => 'page', 'name' => 'Basic page']);
    $this->workflow->getTypePlugin()->addEntityTypeAndBundle('node', 'page');
    $this->workflow->save();

    $node1 = $this->drupalCreateNode(['moderation_state' => 'published']);
    $link1 = MenuLinkContent::create([
      'title' => 'Primary level node',
      'menu_name' => 'main',
      'bundle' => 'menu_link_content',
      'parent' => '',
      'link' => [['uri' => 'entity:node/' . $node1->id()]],
    ]);
    $link1->save();

    $node2 = $this->drupalCreateNode(['moderation_state' => 'published']);
    $node2RevisionId = $node2->getRevisionId();
    $node2->setNewRevision();
    $node2->save();
    $link2 = MenuLinkContent::create([
      'title' => 'Secondary level node',
      'menu_name' => 'main',
      'bundle' => 'menu_link_content',
      'parent' => $link1->getPluginId(),
      'link' => [['uri' => 'entity:node/' . $node2->id()]],
    ]);
    $link2->save();

    // Confirm the menu link of the viewed revision is active.
    $this->drupalGet('node/' . $node2->id() . '/revisions/' . $node2RevisionId . '/view');
    $this->assertMenuActiveTrail([
      'node/' . $node1->id() => $link1->getTitle(),
      'node/' . $node2->id() => $link2->getTitle(),
    ], FALSE);

    // Create a pending revision with no changes.
    $node2->set('moderation_state', 'draft');
    $node2->setNewRevision();
    $node2->save();

    // Confirm the menu link of the latest revision is active.
    $this->drupalGet('node/' . $node2->id() . '/latest');
    $this->assertMenuActiveTrail([
      'node/' . $node1->id() => $link1->getTitle(),
      'node/' . $node2->id() => $link2->getTitle(),
    ], FALSE);
  }

  /**
   * Test that the active trail is added for taxonomy term revisions.
   */
  public function testTaxonomyRevisionMenuTrail() {
    $vocabulary = $this->createVocabulary(['vid' => 'tags']);
    $this->workflow->getTypePlugin()->addEntityTypeAndBundle('taxonomy_term', 'tags');
    $this->workflow->save();

    $term1 = $this->createTerm($vocabulary, ['moderation_state' => 'published']);
    $link1 = MenuLinkContent::create([
      'title' => 'Primary level term',
      'menu_name' => 'main',
      'bundle' => 'menu_link_content',
      'parent' => '',
      'link' => [['uri' => 'internal:/taxonomy/term/' . $term1->id()]],
    ]);
    $link1->save();

    $term2 = $this->createTerm($vocabulary, ['moderation_state' => 'published']);

    $term2RevisionId = $term2->getRevisionId();
    $term2->setNewRevision();
    $term2->save();

    $link2 = MenuLinkContent::create([
      'title' => 'Secondary level term',
      'menu_name' => 'main',
      'bundle' => 'menu_link_content',
      'parent' => $link1->getPluginId(),
      'link' => [['uri' => 'internal:/taxonomy/term/' . $term2->id()]],
    ]);
    $link2->save();

    // Confirm the menu link of the viewed revision is active.
    $this->drupalGet('taxonomy/term/' . $term2->id() . '/revision/' . $term2RevisionId . '/view');
    $this->assertMenuActiveTrail([
      'taxonomy/term/' . $term1->id() => $link1->getTitle(),
      'taxonomy/term/' . $term2->id() => $link2->getTitle(),
    ], FALSE);

    // Create a pending revision with no changes.
    $term2->set('moderation_state', 'draft');
    $term2->setNewRevision();
    $term2->save();

    // Confirm the menu link of the latest revision is active.
    $this->drupalGet('taxonomy/term/' . $term2->id() . '/latest');
    $this->assertMenuActiveTrail([
      'taxonomy/term/' . $term1->id() => $link1->getTitle(),
      'taxonomy/term/' . $term2->id() => $link2->getTitle(),
    ], FALSE);
  }

}
